/* $Id: spp_icmpspoof.c,v 1.1 15/04/2003 19:015:25 */
/* John Papapanos (Internet Systematics Lab) */
/* contact: jpa3nos@lab.epmhs.gr*/

/* Snort Preprocessor Plugin Source File */

/* spp_icmpspoof
 *
 *## Purpose ##:
 * This preprocessor does statefull inspection, detecting spoofed ICMP ECHO Request/Reply
 * packets entering or leaving the protected network(s).
 * Furthermore it detects packets entering or leaving your network that are generated as a
 * result to spoofed packets that did not pass through the Sensor.
 * (eq. If someone outside your network sent a Request to another non existing outside host,
 * by spoofing the IP address of a host inside your network.
 * The Unreachable packet entering your network will generate an alert).
 *
 * spp_icmpspoof aims at detecting attacks such as Decoy Traffic, Os Fingerprinting,
 * Scanning-Network Mapping, DDoS attacks, Covert Channels and others that make use of spoofed 
 * ICMP ECHO packets. 
 *
 
 *## Effect ##
 * It detects packets sent from external hosts to your network, spoofing the
 * Source IP of a host outside your network, a packet sent form an external
 * host to another external host spoofing the IP address of a host from your
 * network and other spoofing scenarios.
 * Outputs an alert and a probable corresponding scenario for the alert generated.
 *
 * Note: It wont detect the case when an internal host sends a spoofed Echo 8
 *       to another internal host using the source ip of an external host, if the
 *       the sensor running snort is able to listen to the internal traffic of
 *       your network in promiscuous mode. In this case if the spoofed host exists
 *       then the spoofing wont be detected, if the spoofed host doesn't exist
 *       then the spoofing will be detected but a wrong scenario case will be
 *       generated.
 *       The best topology is to place snort where it can listen only to the
 *       traffic going in and out of your protected nets.
 *
 * Logs the alerts about the spoofing in a specified log dir/file (or the default)
 * as well as a probable scenario case, describing how the spoofing took place
 * concerning the hosts that took part in the whole process.
 *
 
 *## HOW IT WORKS ##
 * Currently detects only spoofed Echo Requests packets, when it fails
 * to match them with a captured Echo Reply or captured ICMP
 * Unreachables codes 0/1,13, that contain an Echo Request or an Echo Reply
 * packet into their data.
 * As a result of this it will detect spoofed Echo Replies too.
 * To do this the preprocessor stores the required info (src ip , dst ip ,time of packet's arrival)
 * of all the inbound and outbound Icmp Echo Requests packets in 2 identical lists,
 * one for the inbound and one for the outbound Echo Requests.
 * When a packet that is supposed to be an answer to an Echo Request packet arrives,
 * it will be checked, in order for a matching stored Echo Request to be found in the corresponding list.
 *
 * Each of the 2 lists used to store the Echo Requests is a 2 dimensional, circular
 * dynamic linked list.
 * The first dimension of the list holds the senders of the Echo Requests. Every
 * such node has a pointer to a list of the receivers of the Echo Requests this
 * sender has sent to. The receivers form the 2nd dimension of the list.
 * Each sender can be found only once in the list. Each receiver of a sender can
 * also be found only once in the list.
 * Every node on each dimension points to the next and the previous node.
 * The next node pointer, of the last node of each dimension, points to NULL
 * and the previous node pointer of the first node of each dimension, points to the
 * last node of this dimension.
 *
 *
 * ## ARGUMENTS ##
 * It takes up to 3 arguments space character delimited.
 *
 *   	preprocessor icmpspoof:<Protected Net(s)> <Timeout> <Logfile>
 *
 * Protected Net(s)
 *         The network (written in CIDR notation), you wish to check for spoofed
 *         ICMP packets
 *         You can use multiple networks separated by ",".
 *         You can use the var HOME_NET (IP Lists supported), don't use "any".
 * 
 * Timeout (optional)
 *         The number of seconds you wish an icmp echo request packet
 *         to be held stored in the lists. Meaning the maximum number of seconds 
 *         that you expect an Echo Reply or an Icmp Error for an Echo Request.
 *         Bigger value <-> larger time processing, more memory used.
 *         Default value is 3 seconds.
 *
 * Logdir (optional)
 *        The directory/file the preprocessor should log the alerts.
 *        The directory must exist.
 *        The default logging filename is EchoSpoofs and logging dir will
 *        be the pv.logdir given in the command line by -l when running
 *        snort. If no -l is given then the default dir is the one that snort
 *        logs in (eq. /var/log/snort/)
 *
 *--Note--
 * If you want to use the default timeout value and log in a file of your choice
 * then declare a timeout value 0 and the default will be used.
 *
 *--Examples--
 * 1) preprocessor icmpspoof:192.168.100.100/24,192.100.0.10/24 3 /var/log/Spoofs
 *   	Configured like this uses a user defined timeout value and a user
 *       defined logdir.
 *
 * 2) preprocessor icmpspoof:192.168.100.100/24
 *	    Configured like this uses default timeout and default logdir.
 *
 * 3) preprocessor icmpspoof:192.168.100.100/24 0 /var/log/Spoofs
 *	    Configured like this uses default timeout and user defined logdir.
 *
 * 4) preprocessor icmpspoof:192.168.100.100/24 4
 *	    Configured like this uses a user defined timeout value and default
 *       logdir.
 *
 * 5) preprocessor icmpspoof:$HOME_NET 4 /var/log/Spoofs
 *	    Configured like this uses the nets specified in the var HOME_NET,
 *       It can be an IP list , but it can't be "any".
 *
 *
 * ## TODO ##
 * - The plan for the future is to detect more types of spoofed icmp packets and
 *   use more types and codes of icmp error messages to check if spoofing
 *   occurs.
 * 
 * - Improvement of the alert mechanism. More detailed scenarios for spoofed
 *   Icmp Reply packets.
 *
 * - Every alert in the logfile is quite large and maybe the case scenario could
 *   reside in a different file and be pointed by the alert mechanism.
 * 
 * */

/* your preprocessor header file goes here */

#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#ifndef WIN32
    #include <sys/time.h>
    #include <sys/socket.h>
    #include <netinet/in.h>
    #include <arpa/inet.h>
#endif /* !WIN32 */
#include <time.h>

#include "snort.h"
#include "spp_icmpspoof.h"
#include "mstring.h"
#include "log.h"
#include "plugbase.h"
#include "util.h"

#define ICMP_T8C0    8          /* Icmp Echo Request Packet */
#define ICMP_T0C0    0          /* Icmp Echo Reply Packet */
#define ICMP_T3C01t8 3018       /* Icmp Unreachable Host/Net Packet carrying an Icmp Echo Request Packet */
#define ICMP_T3C01t0 3010       /* Icmp Unreachable Host/Net Packet carrying an Icmp Echo Reply Packet */
#define ICMP_T3C13t8 3138       /* Icmp Unreachable Admin. Prohib. Packet carrying an Icmp Echo Request Packet */
#define ICMP_T3C13t0 3130       /* Icmp Unreachable Admin. Prohib. Packet carrying an Icmp Echo Reply Packet */


struct ProtectedNets            /*Nodes that will form the Protected Nets list */
{
	struct in_addr net;         /*IP address of the protected Net transformed from the CIDR notation*/
	uint32_t nmask;				/*The netmask of the net as it is stored in the netmasks table*/
	struct ProtectedNets * nextNet;
};

struct DtgInfo                /*Stores the necessary info about the Icmp Packet*/  
{
	long int Type;            /* Type of the captured Icmp Packet*/ 
    struct in_addr DtgSrc;    /* Packet's source IP address */
    struct in_addr DtgDst;    /* Packet's destination IP address */
    time_t DtgT;              /* Time of arrival of the packet */
};


struct DstNode                /*Srores the required info about the Destination of the Echo t8 packet */  
{
    struct DstNode *PrevDN;   /* Points to the previous Destination node in the list */
    struct in_addr Dst;       /* Packet's destination IP address field */
    time_t T;                 /* Packet's arrival time field*/
    struct DstNode *NextDN;   /* Points to the next Destination node in the list  */
};


struct SrcNode                /*Stores the required info about the Source of the Echo t8 packet */   
{
    struct SrcNode *PrevSN;   /* Points to the previous Source node in the list */
    struct in_addr Src;       /* Packet's Source IP address field */
    struct DstNode *FirstDN;  /* Points to the first node(Destination) in the list */
    struct SrcNode *NextSN;   /* Points to the next Source node in the list */
};


struct RequestList          
{               
    struct SrcNode *InReqs;  /*Points to the first node(source) of inbound requests*/
    struct SrcNode *OutReqs; /*Points to the first node(source) of outbound requests*/
};

struct TrashBin               /*Struct that implements the list containing the recycled nodes */
{
  int SumSN;                    /*Number of the recycled Source Nodes in the Recycle list */
  struct SrcNode *TrSN;         /*Points to the first Source Node in the Recycle list */ 
  int SumDN;                    /*Number of the recycled Destination Nodes in the Recycle list */
  struct DstNode *TrDN;         /*Points to the first Destination Node in the Recycle list */ 
};


/* external globals from rules.c */
extern char *file_name;
extern int file_line;


/*------- Global Vars -------*/
FILE *fpr; 		/*a pointer to the file that will be opened for logging*/ 

char *IcmpLogFile;	/*a pointer to the string that contains the dir/filename of the log file*/
int ReqThreshld;		/*This will store the timeout value for the packets in the list*/

struct DtgInfo Dtg;
struct RequestList Reqlist;
struct TrashBin TrBin;

struct ProtectedNets *ProtectNet;
struct SrcNode *CurList,*CurSN,*DelSN,*InsSN;
struct DstNode *CurDN,*DelDN,*InsDN;



void Get_Dtg_Data(Packet *,int *,int *,time_t *);
void InsertNode_DN();
void InsertNode_SN();
int SearchNode_SN(long int);
int SearchNode_DN(long int);
void EjectCurNode_SN();
void EjectCurNode_DN();
void Prepare_To_Clean_SN();
void Prepare_To_Clean_DN();
void Trash_DN();
void Trash_SN();
void InsertRequest(long int);
int CheckResponse(long int);
void AlertSpoofs(Packet *,int,int,time_t *);
struct ProtectedNets GenProtectedNets (char *);
int IpIsMynet (struct in_addr);
void SplitProtectedNets(char *);
/*void ShowList();*/

/*It could be used in the CleanExitFunction, as it frees all malloced mem.Is it necessary ? */ 
/*void GlobalDelete();*/



/*
 * Function: SetupIcmpSpoof()
 *
 * Purpose: Registers the preprocessor keyword and initialization
 *          function into the preprocessor list.  This is the function that
 *          gets called from InitPreprocessors() in plugbase.c.
 *
 * Arguments: None.
 *
 * Returns: void function
 *
 */
void SetupIcmpSpoof()
{
    /* link the preprocessor keyword to the init function in
       the preproc list */
    RegisterPreprocessor("icmpspoof", IcmpSpoofInit);

#ifdef DEBUG
    printf("Preprocessor: Template is setup...\n");
#endif
}


/*
 * Function: IcmpSpoofInit(u_char *)
 *
 * Purpose: Calls the argument parsing function, performs final setup on data
 *          structs, links the preproc function into the function list.
 *
 * Arguments: args => ptr to argument string
 *
 * Returns: void function
 *
 */
void IcmpSpoofInit(u_char *args)
{
#ifdef DEBUG
    printf("Preprocessor: IcmpSpoof Initialized\n");
#endif

    /*Initializing Structs and pointers*/ 
    Dtg.Type = 8;
    Dtg.DtgSrc.s_addr = 0;
    Dtg.DtgDst.s_addr= 0;
    Dtg.DtgT=0;
    
    Reqlist.InReqs = NULL;
    Reqlist.OutReqs = NULL;
    
    TrBin.SumSN=0;
    TrBin.TrSN=NULL;
    TrBin.SumDN=0;
    TrBin.TrDN=NULL;
    
    
    ProtectNet=NULL;
    CurList=NULL; CurSN=NULL; DelSN=NULL; InsSN=NULL;
    CurDN=NULL; DelDN=NULL; InsDN=NULL;

    /* parse the argument list from the rules file */
    ParseIcmpSpoofArgs(args);

    /* Set the preprocessor function into the function list */
    AddFuncToPreprocList(IcmpSpoofFunction);
/*    AddFuncToCleanExitList(PreprocCleanExitFunction);
    AddFuncToRestartList(PreprocRestartFunction);*/
	 
}


/*
 * Function: ParseIcmpSpoofArgs(char *)
 *
 * Purpose: Process the preprocessor arguements from the rules file and
 *          initialize the preprocessor's data struct.
 *
 * Arguments: args => argument list
 *
 * Returns: void function
 *
 **/
void ParseIcmpSpoofArgs(char *args)
{
    char **toks;
	int numToks,i;
	struct ProtectedNets *pr_Net=NULL;
	
	char *LogFile;
    
	LogMessage("spp_icmpspoof configuration :\n");
    
	if (args==NULL || *args=='\0')
	{
        FatalError("spp_icmpspoof: You should at list define protected NET(S)in %s(%d) \n",file_name,file_line);
    }
    else
    {
        toks=mSplit(args," ",3,&numToks,'\\');
        
        SplitProtectedNets(toks[0]);
						
		LogMessage("    Protected Nets :\n");
		pr_Net=ProtectNet;
		while(pr_Net)
		{
			LogMessage("                    - %s\n",inet_ntoa(pr_Net->net));
			pr_Net=pr_Net->nextNet;
		}
		
		if((!toks[1])||(!strncmp(toks[1],"0",sizeof(char))))
        {
            ReqThreshld = 3;
			LogMessage("    Default Timeout : %d seconds\n",ReqThreshld);
        }
        else
        {
            if(!(ReqThreshld=atoi(toks[1])))
                FatalError("spp_icmpspoof:Wrong Value for Timeout arg %s in %s(%d)\n",toks[1],file_name,file_line);
                LogMessage("    Timeout : %d\n",ReqThreshld);
        }
            

        if(!toks[2]) /*If no Logdir arg was given use the default*/ 
        {
            
            LogFile ="Echospoofs" ;
            /*Checking if logdir given in cmd with -l ends with '/' */
            if (pv.log_dir +(strlen(pv.log_dir)-1) == strrchr(pv.log_dir,'/'))
            {
                IcmpLogFile=(char *) malloc (strlen(pv.log_dir)+strlen(LogFile)+1);
                if (IcmpLogFile == NULL)
                    FatalError("spp_icmpspoof: Can't allocate Memory for filename \n",file_name,file_line);
                strcpy(IcmpLogFile,pv.log_dir);
                strcat(IcmpLogFile,LogFile);
            }
            else
            {
                IcmpLogFile=(char *) malloc (strlen(pv.log_dir)+strlen(LogFile)+2);
                if (IcmpLogFile == NULL)
                    FatalError("spp_icmpspoof: Can't allocate Memory for filename \n",file_name,file_line);
                strcpy(IcmpLogFile,pv.log_dir);
                strcat(IcmpLogFile,"/");
                strcat(IcmpLogFile,LogFile);
            }
           LogMessage("    Default Log Dir/File : %s\n",IcmpLogFile);
        }
        else
        {
            LogFile=toks[2];
            IcmpLogFile=LogFile;                
            LogMessage("   Log Dir/File : %s\n",IcmpLogFile);
        }
        
                       
        for(i = 0; i < numToks; i++)
        {
            free(toks[i]);
        }
        
                    
        fpr=fopen(IcmpLogFile,"a");
        if (fpr==NULL)
        {
            FatalError("\nspp_icmpspoof: Error Opening ICMP Spoof File -\n\
               Check logfile path/name in %s(%d)\n ",file_name,file_line);
            
        }
	
    }
}
                    

/*
 * Function: IcmpSpoofFunction(Packet *)
 *
 * Purpose: Perform the preprocessor's intended function.Each time a packet arrives, only
            the wanted icmp packets will be processed.Inserts all Echo Request packets in
            the apropriate list and checks the list for spoofing when an Echo Reply or 
            an Unreachable icmp packet arrives.
 *
 * Arguments: p => pointer to the current packet data struct
 * 
 * Returns: void function
 *
 */
void IcmpSpoofFunction(Packet *p)
{
  
	int action=0;       /*Gets it's value from Get_Dtg_Data() and determines if a search 
                          or an insertion to the list ,will take place*/
    int fix_Curlist;    /*Gets it's value from Get_Dtg_Data() and determines wich list  
                          (inbound or outbound) will be used.*/
	int alertaction=0;  /*Determines if an alert should be raised */
	long int limit;	 	/*It's the smaller time an Echo Request should have in the list*/
    time_t arrvtime;    /*Time of the packet's arrival*/
	
  
 /*Dont't process fragments and non Icmp packets */
  if (!(p->frag_flag) && (PacketIsICMP(p))) 
  {
	 /*Dont't process packets when src and dst ip don't belong to one of the protected nets */
     if ((IpIsMynet(p->iph->ip_src))||(IpIsMynet(p->iph->ip_dst))) 
     {
	 	  Get_Dtg_Data(p,&action,&fix_Curlist,&arrvtime);
		  
         /*Initialize */
          limit=Dtg.DtgT-ReqThreshld;
		  CurSN = CurList;
          if(CurSN==NULL)
              CurDN=NULL;
          else
              CurDN=CurSN->FirstDN;
          
		  DelSN = NULL;
		  DelDN = NULL;
		  
		  switch (action)
		  {
			  
              case 1:
                  /*Do Insertion In the list */
				  InsertRequest(limit);
				  
				  /*Delete old nodes from the list*/
                  if (DelDN != NULL)
					  Trash_DN();
				  
				  if (DelSN != NULL)
					  Trash_SN();
                  
                  /*Set the starting point of the list after processing */
                  if (fix_Curlist==1)
                      Reqlist.InReqs=CurList;
                  if (fix_Curlist==2)
                      Reqlist.OutReqs=CurList;
				  
				  break;
			  case 2:
                  /*Check the list for spoofing*/
				  alertaction=CheckResponse(limit);
                  /*Raise the alert*/
                  /*An alert can also be raised if an Unreach. 0/1 packet is detected containing
                    an Echo Reply packet even if the matching Echo Request is found in the List,
                    because it means that you answer to someone who sent you a question but 
                    he doesn't exist.*/
				  if((alertaction==1)||((alertaction==0)&&(Dtg.Type==ICMP_T3C01t0)))
					  AlertSpoofs(p,alertaction,fix_Curlist,&arrvtime);
                  
				  /*Delete old nodes from the list*/
				  if (DelDN != NULL)
					  Trash_DN();
				  
				  if (DelSN != NULL)
					  Trash_SN();
                  
                  /*Set the starting point of the list after processing */
                  if (fix_Curlist==1)
                      Reqlist.InReqs=CurList;
                  if (fix_Curlist==2)
                      Reqlist.OutReqs=CurList;
                  
				  break;
			  default:
				  break;
		  }
     }
  } 
}

/*
 * Function: SplitProtectedNets()
 *
 * Purpose: Takes the first arg from the rules file containing the 
 *          protected nets, splits them , and puts them in the 
 *          Protected Nets List after calculating the real ip address
 *			extracted from the CIDR notation.
 *
 * Arguments: A char pointer pointing to the string containing all 
 *            the protected nets as it was created in ParseIcmpSpoofArgs().
 *
 * Returns: void function
 *
 */
void SplitProtectedNets(char *netargs)
{
	struct ProtectedNets *newPrNet,*curPrNet=NULL;
	char **net_toks;
	int net_numToks;
	int i,given_nets=1;
	
	/*Suport for IP lists*/
	if ((netargs[0]=='[') && (netargs[ strlen(netargs)-1 ]==']'))
	{
		netargs=&netargs[1];
		netargs[ strlen(netargs)-1 ]='\0';
	}
	
	/*Count how many nets were given in order to use it in mSplit() afterwards*/
    for	(i=0;i<strlen(netargs); i++)
	{
		if(netargs[i]==',')
		{
			given_nets++;
			
		}
	}
	
	/*Split nets in net/mask pairs */
    net_toks=mSplit(netargs,",",given_nets,&net_numToks,'\\');
	
	/*For every net/mask pair calculate the real net ip from the CIDR notation
      and store it in a node in the Protected Nets list*/
    for(i=0;i<net_numToks;i++)
	{
		newPrNet=(struct ProtectedNets *) malloc (1*sizeof(struct ProtectedNets));
		if(newPrNet==NULL)
			FatalError("\nspp_icmpspoof: Error Can't Allocate memory for Protected Nets\
						%s(%d)\n",file_name,file_line);
		*newPrNet=GenProtectedNets(net_toks[i]);
		
		if(i==0)
			ProtectNet=newPrNet;
		else
			curPrNet->nextNet=newPrNet;
		
		curPrNet=newPrNet;
		free(net_toks[i]);
	}
	
		
}

/*
 * Function: SplitProtectedNets()
 *
 * Purpose:  Detects if an IP address belongs to one of the Protected Nets.           
 *
 * Arguments: An in_addr struct containing the ip address to be checked         
 *
 * Returns: If the ip belongs to one of the protected nets returns 1, else 0
 *
 */
int IpIsMynet (struct in_addr ipaddr)
{
	struct ProtectedNets *curPrNet=ProtectNet;
		
	while(curPrNet)
	{
		if((ipaddr.s_addr & curPrNet->nmask) == curPrNet->net.s_addr)
		{
			return 1;
		}
		
		curPrNet=curPrNet->nextNet;
	
	}
	return 0;
}

/*
 * Function: GenProtectedNets()
 *
 * Purpose:  Calculates the ip of the protected net from a string containing
 *           a pair of net/mask in CIDR notation. It is a slightly modified version
 *           of the GenHomenet() function (snort.c).
 *
 * Arguments: A char pointer to the string containing the net/mask pair in CIDR notation.
 *
 * Returns: A ProtectedNets Node containing the calculated net ip and the netmask used.
 *
 */
struct ProtectedNets GenProtectedNets (char *net_tok)
{
    
    struct ProtectedNets PrNet;
	struct in_addr getnet;		/* place to stick the local network data */
    char **toks;                /* dbl ptr to store mSplit return data in */
    int numToks;                /* number of tokens mSplit returns */
    int nmask;                        /* temporary netmask storage */
    int i;

    /* break out the CIDR notation from the IP address */
    toks = mSplit(net_tok, "/", 2, &numToks, 0);

    if(numToks > 1)
    {
        /* convert the CIDR notation into a real live netmask */
        nmask = atoi(toks[1]);

        if((nmask > 0) && (nmask < 33))
        {
            PrNet.nmask = netmasks[nmask];
        }
        else
        {
            FatalError("\nspp_icmpspoof: Bad CIDR block [%s:%d], 1 to 32 please!\n",toks[1],nmask);
        }
    }
    else
    {
        FatalError("\nspp_icmpspoof: No netmask specified for home network! :%s (%d)\n",file_name,file_line);
    }

    PrNet.nmask = htonl(PrNet.nmask);


    /* convert the IP addr into its 32-bit value */
    if((getnet.s_addr = inet_addr(toks[0])) == -1)
    {
        FatalError("\nspp_icmpspoof: Homenet (%s) didn't x-late, WTF? %s (%d)\n",toks[0],file_name,file_line);
    }
    else
    {
        /* set the final homenet address up */
        PrNet.net.s_addr = ((u_long) getnet.s_addr & PrNet.nmask);
    }
	

    for(i = 0; i < numToks; i++)
    {
        free(toks[i]);
    }
	
	PrNet.nextNet=NULL;
	return PrNet;
}


/*
 * Function: Get_Dtg_Data()
 *
 * Purpose:  This function filters a captured icmp packet extracting from it the required 
 *           info wich stores it in an appropriate struct(DtgInfo).
 *           It also determines from the captured packet, which action(Insert,Check) 
 *           will occur, and which list(inbound,outbound) will be used.  
 *           
 * Arguments: A char pointer to the captured packet,a pointer to the action to be taken,
 *            a pointer to an int indicating the list that will be used ,a pointer to 
 *            the time_t struct that will hold the packet's captured time.
 *
 * Returns: All the values that need to be returned are passed into the function by reference.
 *
 */
void Get_Dtg_Data(Packet *p,int *action,int *fix_Curlist,time_t * captime)
{
	
	time(captime);  /* Get the time*/
	    
	switch (p->icmph->type)
          {
              case ICMP_ECHO:
                  /*Don't accept Echo 8 packets that both the src,dst ips belong
                    to one or more of the Protected Nets.This is done so internal traffic 
                    will be ignored for less overload.Spoofing is only detected from
                    inbound and outbound traffic, even if an internal host is sending
                    spoofed packets to another internal host(if only he is spoofing an 
                    external's host ip)*/
                  if(!(IpIsMynet(p->iph->ip_src) && IpIsMynet(p->iph->ip_dst)))
                  {
                      /*If an internal host is sending the packet*/                      
                      if (IpIsMynet(p->iph->ip_src))
                      {
                          CurList=Reqlist.OutReqs; /*Outbound list will be used*/
                          *fix_Curlist=2;
					  }
                      else 
                      {
                          CurList=Reqlist.InReqs; /*Inbound list will be used*/
                          *fix_Curlist=1;
					  }
                      
                      /*Fill the Dtg struct with the needed info*/
                      Dtg.Type = ICMP_T8C0;
                      Dtg.DtgSrc = p->iph->ip_src;
                      Dtg.DtgDst = p->iph->ip_dst;
                      Dtg.DtgT = *captime;
                      
                      *action = 1; /*Insertion will take place */
                  }
                  else
                      *action = 0; /*Do Nothing*/
                  
                  break;
                                    
              case ICMP_ECHOREPLY:
                  /*Don't accept Echo 0 packets that both the src,dst ips belong
                    to one or more Protected Nets*/
                  if(!(IpIsMynet(p->iph->ip_src) && IpIsMynet(p->iph->ip_dst)))
                  {
                      /*If an internal host is sending the packet*/                      
                      if (IpIsMynet(p->iph->ip_src))
                      {                      
                          CurList=Reqlist.InReqs;  /*Inbound list will be used*/
                          *fix_Curlist=1;
                      }
                      else
                      {                      
                          CurList=Reqlist.OutReqs; /*Outbound list will be used*/
                          *fix_Curlist=2;
                      }
                      /*Fill the Dtg struct with the needed info*/
                      Dtg.Type = ICMP_T0C0;
                      Dtg.DtgSrc = p->iph->ip_dst; /*Rotate src and dst so it looks..*/
                      Dtg.DtgDst = p->iph->ip_src; /*like an Echo 8,for the searching*/  
                      Dtg.DtgT = *captime;
                      
                      *action=2; /*Cheking for spoofing will take place*/
                  }
                  else
                      *action=0; /*Do Nothing*/
                  
                  break;
                   
              case ICMP_DEST_UNREACH:
				   /*Accept only Icmp Unreachables the preprocessor currently can handle */				  
                   if ((p->icmph->code!=ICMP_NET_UNREACH)&&(p->icmph->code!=ICMP_HOST_UNREACH)&&(p->icmph->code!=ICMP_PKT_FILTERED))
                   {
                       *action = 0; /* Do nothing */
				   }
				   else
                   {
                       /*Accept packet only if the inside(original) packet is icmp*/
				        if(p->orig_iph != NULL && p->orig_icmph != NULL)
                        {
                           /*Dont't accept Unreachables that src,dst ips of the original packets
                             both belong (or both don't belong) to one or more of the protected nets */ 
                           if((IpIsMynet(p->orig_iph->ip_src)||IpIsMynet(p->orig_iph->ip_dst))&&((!IpIsMynet(p->orig_iph->ip_src))||(!IpIsMynet(p->orig_iph->ip_dst))))
                           {
                               *action = 2; /*Cheking for spoofing will take place*/
                               switch (p->orig_icmph->type)
                               {
                                   case ICMP_ECHO:
                                       /*If an internal host is sending the packet*/                      
                                       if (IpIsMynet(p->orig_iph->ip_src))
									   {
										   CurList = Reqlist.OutReqs; /*Outbound list will be used*/
										   *fix_Curlist=2;
									   }
									   else
									   {
										   CurList=Reqlist.InReqs;  /*Inbound list will be used*/
										   *fix_Curlist=1;
									   }
                                       /*Fill the Dtg struct with the needed info*/
                                       Dtg.DtgSrc = p->orig_iph->ip_src;
                                       Dtg.DtgDst = p->orig_iph->ip_dst;
                                       Dtg.DtgT = *captime;
                                       if ((p->icmph->code == 0) || (p->icmph->code == 1))
                                           Dtg.Type = ICMP_T3C01t8;
                                       else
                                           Dtg.Type = ICMP_T3C13t8;
                                       
                                       break;
                                       
                                   case ICMP_ECHOREPLY:
                                       /*If an internal host is sending the packet*/                                                             
									   if (IpIsMynet(p->orig_iph->ip_src))
									   {
										   CurList=Reqlist.InReqs; /*Inbound list will be used*/
										   *fix_Curlist=1;
									   }
									   else
									   {
										   CurList=Reqlist.OutReqs; /*Outbound list will be used*/
										   *fix_Curlist=2;
									   }
                                                             
                                       /*Fill the Dtg struct with the needed info*/
									   Dtg.DtgSrc = p->orig_iph->ip_dst;
                                       Dtg.DtgDst = p->orig_iph->ip_src;
                                       Dtg.DtgT = *captime;
                                       
                                       if ((p->icmph->code == 0) || (p->icmph->code == 1))
                                           Dtg.Type = ICMP_T3C01t0;
                                       else
                                           Dtg.Type = ICMP_T3C13t0;
                                       
                                       break;
                                       
                                   default:
                                       *action=0;  /*Do Nothing*/
                                       break;
                                }

                           }
                           else
                               *action=0;
                            
                        }
                        else
                        {
                            *action=0;
                        }
                   }
                   break;
              default :
				   *action=0;
				   break;
          }
};
	

/*
 * Function: InsertRequest()
 *
 * Purpose:  This function perfoms the Insertion of the Echo 8 packet's needed info into the list.
 *           It will also prepare the list to be cleaned from old packets later on .
 *
 * Arguments: The long int limit variable which holds the smaller(oldest) time 
 *            an Echo Request can have in the list, so it can stay in the list.
 *              
 * Returns: Nothing, it works on global vars.
 *
 */
void InsertRequest(long int limit)
{
	int flagSN=0;
	int flagDN=0;
			
    /*If the list is empty*/
    if (CurSN==NULL) 
	{
		InsertNode_SN();
	}
    else
    {
		/*Go to the last node in the list(the last inserted)*/
        CurSN=CurSN->PrevSN;
        CurDN=CurSN->FirstDN;
        
        /*Check if the last node is within the time limits
          if not , then all nodes need to be deleted*/  
        if (CurDN->PrevDN->T < limit)
        {
            DelSN=CurList;
            CurList=NULL;
            DelSN->PrevSN=NULL;
            CurSN=NULL;
            CurDN=NULL;
			InsertNode_SN();
        }
        else
        {
            /*Check every node from 1st to last to determine if they have valid times 
              and if the same src ip has previously sent an Echo 8 and is
              already inserted in the list*/
            CurSN=CurList;
            flagSN=SearchNode_SN(limit);
            
            if (flagSN == 1)       /*if the source ip has previously sent an Echo 8..*/
                EjectCurNode_SN(); /*this node will be transfered at the end of the list having..*/
            else                   /*the new time*/ 
                CurDN=NULL;
            
            InsertNode_SN();
            Prepare_To_Clean_SN();
            
            if(flagSN == 1) /*Check if the already inserted src ip has sent an Echo 8..*/
            {               /*to this dst ip before*/
                CurDN=CurSN->FirstDN;
                flagDN=SearchNode_DN(limit);
                
                if (flagDN==1)         /*if the dst ip is already in the list of the hosts..*/ 
                    EjectCurNode_DN(); /*this src ip has sent an Echo 8, then it will be..*/ 
                                       /*transfered at the end of the list having the new time*/ 
                InsertNode_DN();
                Prepare_To_Clean_DN();
            }
        }
    }
    
    /*fill the node with the needed data*/
    CurSN->Src = Dtg.DtgSrc;
    CurDN->Dst = Dtg.DtgDst;
    CurDN->T = Dtg.DtgT;
    
};


/*
 * Function: CheckResponse()
 *
 * Purpose:  This function checks if the non Icmp Echo 8 packets that arrive,match 
 *           with an Echo 8 packet from the list.If not,then it will blow the whistle
 *           for an alert to be raised.It will also prepare the list to be cleaned 
 *           later on from old packets. It works almost the same like the 
 *           InsertRequest() function.
 *
 * Arguments: The long int limit variable which holds the smaller(oldest) time 
 *            an Echo Request can have, so it can stay in the list.
 *
 * Returns: It returns 1 if an alert should be raised, 0 if not.
 *
 */
int CheckResponse(long int limit)
{
	int alert=0;
	int flagSN=0;
	int flagDN=0;
	
	if (CurSN == NULL)
    {
		alert=1;
    }
    else
    {
        CurSN = CurSN->PrevSN;
        CurDN = CurSN->FirstDN;
        
        if (CurDN->PrevDN->T < limit)
        {
            DelSN = CurList;
            CurList=NULL;
            DelSN->PrevSN=NULL;
            CurSN=NULL;
            CurDN=NULL;
            alert=1;
        }
        else
        {
            CurSN = CurList;
            flagSN = SearchNode_SN(limit);
            
            if (flagSN !=1)
            {
                alert=1;
            }
            else
            {
                CurDN =CurSN->FirstDN;
                flagDN = SearchNode_DN(limit);
                Prepare_To_Clean_DN();
                
                if (flagDN !=1)
                {
                    alert=1;
                }
            }
        }
    }
   return(alert); 
};


/*
 * Function: SearchNode_SN()
 *
 * Purpose:  This function checks all the nodes of the senders of the Echo 8
 *           packets one by one to find out if they are to old and if the new
 *           sender has sent an Echo 8 before and is already in the list.   
 *           
 * Arguments: The long int limit variable which holds the smaller(oldest) time 
 *            an Echo Request can have, so it can stay in the list.
 *
 * Returns: 1 if the sender is already in the list, 0 if not.
 *
 */
int SearchNode_SN(long int limit)
{
    while (CurSN !=NULL) /*Checks the list from start to end*/
    {
        CurDN = CurSN->FirstDN;
        
        if (CurDN->PrevDN->T >= limit) /*check the time of the..*/  
        {                              /*Echo 8 he has sent*/ 
            if (CurSN->Src.s_addr == Dtg.DtgSrc.s_addr)
            {
                return 1; /*the host is found in the list*/
            }
        }
        else
            DelSN=CurSN;
        
        CurSN = CurSN->NextSN;
    }
	return 0;
};


/*
 * Function: SearchNode_DN()
 *
 * Purpose:  This function checks all the nodes of the recievers of the Echo 8
 *           packets of a given sender one by one to find out if they are to 
 *           old and if the new reciever has recieved an Echo 8 before from this 
 *           sender and is already in the list.   
 *           
 * Arguments: The long int limit variable which holds the smaller(oldest) time 
 *            an Echo Request can have, so it can stay in the list.
 *
 * Returns: 1 if the reciever is already in the list, 0 if not.
 *
 */
int SearchNode_DN(long int limit)
{
    while (CurDN !=NULL)
    {
        if (CurDN->T >= limit) /*check the time of the..*/  
        {                      /*Echo 8 he has recieved*/ 
            if (CurDN->Dst.s_addr==Dtg.DtgDst.s_addr)
            {
                return 1; /*the reciever is found in the list*/
            }
        }
        else
            DelDN=CurDN;
        
        CurDN = CurDN->NextDN;
    }
	return 0;
};


/*
 * Function: EjectCurNode_SN()
 *
 * Purpose:  This function ejects a node from the list so it can be placed
 *           later on at the end of the list.The node to be ejected was pointed
 *           out by the SearchNode_SN function.
 *           It works on nodes holding the senders' ips.
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void EjectCurNode_SN()
{
    /*if the node is not already at the end of the list*/
    if (CurSN->NextSN != NULL) 
    {
        CurSN->NextSN->PrevSN = CurSN->PrevSN;
        /*if the node is not the first in the list*/
        if (CurSN !=CurList)
            CurSN->PrevSN->NextSN = CurSN->NextSN;
        else
            CurList = CurSN->NextSN; 
        
        CurSN->NextSN = NULL;
        CurSN->PrevSN = NULL;
    }
};



/*
 * Function: EjectCurNode_DN()
 *
 * Purpose:  This function operates like EjectCurNode_SN,but works on 
 *           nodes holding the recievers' ips.
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void EjectCurNode_DN()
{
    if (CurDN->NextDN != NULL)
    {
        CurDN->NextDN->PrevDN = CurDN->PrevDN;
        if (CurDN !=CurSN->FirstDN)
            CurDN->PrevDN->NextDN = CurDN->NextDN;
        else
            CurSN->FirstDN = CurDN->NextDN;
        
        CurDN->NextDN = NULL;
        CurDN->PrevDN = NULL;
    }
};


/*
 * Function: InsertNode_SN()
 *
 * Purpose:  This function inserts a node into the list.The insertion is 
 *           always done at the end of the list.The new node is either :
 *             - An existing node that needs to be overwriten and be 
 *           	 placed at the end(in this case this node is pointed after 
 *           	 the execution of the SearchNode_SN() function),
 *             - A node from the reclycle list,
 *             - A node created by a new malloc() if the recycle list is
 *               empty.
 *           The last too cases take place only if the sender is not already
 *           found in the list by SearchNode_SN().
 *           It works on nodes holding the senders' ips.   
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void InsertNode_SN()
{
    if (CurSN == NULL) /*The sender doesn't already exist in the list*/
    {
        if (TrBin.TrSN !=NULL) /*If the recycle list is not empty..*/
        {                      /*use a node from there*/ 
            CurSN = InsSN;
            if (InsSN != TrBin.TrSN) /*if there are more then one..*/  
            {                        /*nodes in the recycle list*/
                InsSN = InsSN->PrevSN;
                InsSN->NextSN = NULL;
                CurSN->PrevSN = NULL;
            }
            else
            {
                InsSN = NULL;
                TrBin.TrSN = NULL;
            }
            
            TrBin.SumSN = TrBin.SumSN--;
        }
        else /*create a new node*/
        {
            CurSN = (struct SrcNode *) malloc (1*sizeof(struct SrcNode));
			if (CurSN == NULL)
				FatalError("spp_icmpspoof: Not Enough Memory For Use \n",file_name,file_line);
                                   
            /*Initialize the new created node*/
            CurSN->PrevSN=NULL;
            CurSN->FirstDN=NULL;
            CurSN->Src.s_addr=0;
            CurSN->NextSN=NULL;
        }
        
        InsertNode_DN(); /*Do the same for the recievers*/
    }
    
    /*Put the node at the end of the list*/
    if (CurSN->PrevSN == NULL)
    {
        if (CurList == NULL) /*The list is empty*/
        {
            CurList = CurSN;
            CurList->PrevSN = CurSN;
        }
        else
        {
            CurSN->PrevSN = CurList->PrevSN;
            CurList->PrevSN->NextSN = CurSN;
            CurList->PrevSN = CurSN;
        }
    }
};

/*
 * Function: InsertNode_DN()
 *
 * Purpose:  This function inserts a node into the list.It operates like 
 *           the InsertNode_SN() function , but It works on nodes holding 
 *           the recievers' ips.   
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void InsertNode_DN()
{
    if (CurDN == NULL)
    {
        if (TrBin.TrDN !=NULL)
        {
            CurDN = InsDN;
            if (InsDN != TrBin.TrDN)
            {
                InsDN = InsDN->PrevDN;
                InsDN->NextDN = NULL;
                CurDN->PrevDN = NULL;
            }
            else
            {
                InsDN = NULL;
                TrBin.TrDN = NULL;
            }
            
            TrBin.SumDN = TrBin.SumDN--;
        }
        else
        {
            CurDN = (struct DstNode *) malloc (1*sizeof(struct DstNode));
			if(CurDN == NULL)
				FatalError("spp_icmpspoof: Not Enough Memory for Use \n",file_name,file_line);
                
            CurDN->PrevDN=NULL;
            CurDN->Dst.s_addr=0;
            CurDN->T=0;
            CurDN->NextDN=NULL;
        }
        
    }
    
    if (CurDN->PrevDN == NULL)
    {
        if (CurSN->FirstDN == NULL)
        {
            CurSN->FirstDN = CurDN;
            CurDN->PrevDN = CurDN;
        }
        else
        {
            CurDN->PrevDN = CurSN->FirstDN->PrevDN;
            CurSN->FirstDN->PrevDN->NextDN = CurDN;
            CurSN->FirstDN->PrevDN = CurDN;
            CurDN->NextDN = NULL;
        }
    }
};
        

/*
 * Function: Prepare_To_Clean_SN()
 *
 * Purpose:  This function prepares the list so it can be cleaned from old
 *           nodes later on.
 *           It works on senders' nodes.
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void Prepare_To_Clean_SN()
{
    /*if there is something to clean*/
    if  (DelSN != NULL)
    {
        DelSN -> NextSN->PrevSN = CurList ->PrevSN;
        CurList->PrevSN->NextSN = CurList;
        CurList->PrevSN = NULL;
        CurList=DelSN->NextSN;
        DelSN->NextSN=NULL;
        DelSN=CurList->PrevSN->NextSN;
        CurList->PrevSN->NextSN=NULL;
    }
};


/*
 * Function: Prepare_To_Clean_DN()
 *
 * Purpose:  This function prepares the list so it can be cleaned from old
 *           nodes later on.
 *           It works on recievers' nodes.
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void Prepare_To_Clean_DN()
{
    if  (DelDN != NULL)
    {
        DelDN->NextDN->PrevDN = CurSN->FirstDN->PrevDN;
        CurSN->FirstDN->PrevDN = NULL;
        CurDN->NextDN = CurSN->FirstDN;
        CurSN->FirstDN=DelDN->NextDN;
        DelDN->NextDN=NULL;
        DelDN=CurDN->NextDN;
        CurDN->NextDN=NULL;
    }
};


/*
 * Function: Trash_DN()
 *
 * Purpose:  This function cleans the list from old packets.A node that needs
 *           to be deleted from the list is being transfered to the recycle
 *           list. If the recycle list is full then the node is deleted by
 *           freeing the memory space it consumes.
 *           The idea of the recycle list is to recycle a certain number of
 *           nodes so i wont have to allocate small amounts of memory again and
 *           again.
 *           It works on recievers' nodes.
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void Trash_DN()
{
	int maxDN=50; /*Maximum number of nodes that the recycle list can hold*/
                  /*every node has a size o 4x4=16 bytes*/
    if (InsDN != NULL)
    {
        InsDN->NextDN = DelDN; /*the Recycle list is not empty*/
        DelDN->PrevDN =InsDN;
    }
    else
    {
        TrBin.TrDN = DelDN;
        InsDN = DelDN;
        DelDN->PrevDN =NULL;
    }
    
    while (DelDN->NextDN != NULL) /*start puting all nodes that need to..*/
    {                             /*be cleaned in the recycle list,until..*/  
        DelDN = DelDN->NextDN;    /*the maximum number is reached*/
        if (TrBin.SumDN < maxDN)
        {
            InsDN = DelDN->PrevDN;
            TrBin.SumDN++;
        }
        else
            free(DelDN->PrevDN);/*Delete completely the nodes that don't fit..*/
    }                           /*into the recycle list*/
    
    if (TrBin.SumDN < maxDN)
    {
        InsDN = DelDN;
        TrBin.SumDN++;
    }
    else
        free(DelDN);
    
    InsDN->NextDN = NULL;
    DelDN=NULL;
};
  

/*
 * Function: Trash_SN()
 *
 * Purpose:  This function operates like the Trash_DN function but it works
 *           on senders' nodes.
 *           
 * Arguments: None
 *
 * Returns: Nothing.
 *
 */
void Trash_SN()
{
	int maxSN=10;
    if (InsSN != NULL)
    {
        InsSN->NextSN = DelSN;
        DelSN->PrevSN =InsSN;
    }
    else
    {
        TrBin.TrSN = DelSN;
        InsSN = DelSN;
        DelSN->PrevSN =NULL;
    }
    
    while (DelSN->NextSN != NULL)
    {
        DelDN=DelSN->FirstDN;
        DelSN->FirstDN=NULL;
        DelSN = DelSN->NextSN;
        
        if (TrBin.SumSN < maxSN)
        {
            InsSN = DelSN->PrevSN;
            TrBin.SumSN++;
        }
        else
            free(DelSN->PrevSN);
        
        Trash_DN();
    }
    
    DelDN=DelSN->FirstDN;
    DelSN->FirstDN=NULL;
    
    
    if (TrBin.SumSN < maxSN)
    {
        InsSN = DelSN;
        TrBin.SumSN++;
    }
    else
        free(DelSN);
    
    Trash_DN();    
    InsSN->NextSN = NULL;
    DelSN=NULL;
};


/*
 * Function: AlertSpoofs()
 *
 * Purpose:  This function logs the alert in the logging dir/file followed 
 *           by an explanation of the way the spoofing occured.
 *           An alert is raised when there wasn't found a matching Echo 8 
 *           packet in the list , for a captured Echo 0 or Unreachable
 *           packet.The only case that an alert is raised even if a
 *           matching Echo 8 was found in the list, is when the packet 
 *           that is being inspected is an Unreachable code 0/1 and 
 *           contains an Echo 0 as an original packet. 
 *           This is happening because host B is sending a reply to host A 
 *           for an Echo 8 that host A is supposed to have sent to host B,
 *			 but host A doesn't exist.
 *           
 * Arguments: A pointer to the packet captured, the alert result returned
 *            by the CheckResponse() function, the list(inbound,outbound) 
 *            that was used to check the spoofing,the time that the packet 
 *            arrivved.
 *
 * Returns: Nothing.
 *
 */

void AlertSpoofs(Packet *p,int alert,int CurrentList,time_t *alerttime)
{
	char * Scenario=NULL;/*it will point to the string containing the scenario*/
	
	
	if(CurrentList == 1) /*INBOUND list was used*/ 
	{
		if(alert==0) /*&& (Dtg.Type==ICMP_T3C01t0)*/
			Scenario="   * From EXTERNAL Host(ATTACKER)\n   *   To INTERNAL Host(TARGET)\n   * Spoofing IP Address of EXTERNAL Host(SPOOFED)";
		else
			Scenario="   * From INTERNAL Host(ATTACKER)\n   *   To INTERNAL Host(TARGET)\n   * Spoofing IP Address of EXTERNAL Host(SPOOFED)";
	}
	else				/*OUTBOUND list was used*/
	{
		if(alert==0) /*matching Echo 8 was found in the list*/
			Scenario="   * From INTERNAL Host(ATTACKER)\n   *   To EXTERNAL Host(TARGET)\n   * Spoofing IP Address of INTERNAL Host(SPOOFED)";
		else
			Scenario="   * From EXTERNAL Host(ATTACKER)\n   *   To EXTERNAL Host(TARGET)\n   * Spoofing IP Address of INTERNAL Host(SPOOFED)";
	}
		
	
	
        /*Print the packet*/
		fprintf(fpr,"\n");
		fprintf(fpr,"******************* SPOOFED PACKET DETECTED *********************\n");
		fprintf(fpr,"%s",ctime(alerttime));
		fprintf(fpr,"======< PACKET THAT GENERATED THE ALERT >======\n");
		fprintf(fpr,"|------------- < IP HEADER > -------------|\n");
        PrintIPHeader(fpr,p);
		fprintf(fpr,"|------------- <ICMP HEADER> -------------|\n"); 
		PrintICMPHeader(fpr,p);
        fprintf(fpr,"============== < END Of Packet > ==============\n"); 
	
		/*Print the scenario*/
        if (alert==1)
		{
			fprintf(fpr,"!!! No Matching ICMP ECHO REQUEST Packet Seen For This Packet !!! \n ");
			fprintf(fpr,"\n");
			fprintf(fpr,"==============< SCENARIO >============== \n");
			fprintf(fpr,"** Probably Spoofed ICMP ECHO REQUEST Packet Sent\n");
			fprintf(fpr,"%s\n",Scenario);
			fprintf(fpr,"   * - TARGET  Host %s \n",inet_ntoa(Dtg.DtgDst));
			fprintf(fpr,"   * - SPOOFED Host %s \n",inet_ntoa(Dtg.DtgSrc));
			
			if((Dtg.Type == ICMP_T0C0)||(Dtg.Type == ICMP_T3C13t0))
				fprintf(fpr,"** OR A SPOOFED ICMP ECHO REPLY Packet Sent \n");
			else if(Dtg.Type == ICMP_T3C01t0) 
            {
                fprintf(fpr,"   * AND Spoofed Host Doesn't Exist\n");
                fprintf(fpr,"** OR A SPOOFED ICMP ECHO REPLY Packet Sent \n");
				fprintf(fpr,"   * AND Spoofed Host Doesn't Exist\n");
            }
			else if(Dtg.Type == ICMP_T3C01t8) 
				fprintf(fpr,"   * AND Target Host Doesn't Exist\n");
            fprintf(fpr,"=================< END >=================\n");
            fprintf(fpr,"\n");
		}
		else
		{
			
            fprintf(fpr,"!!! The matching Echo Request for the Echo Reply(Original Packet) Exsisted !!!\n");
            fprintf(fpr,"\n");
			fprintf(fpr,"==============< SCENARIO >==============  \n");
			fprintf(fpr,"** Probably Spoofed ICMP ECHO REQUEST Packet Sent\n");
			fprintf(fpr,"%s\n",Scenario);
			fprintf(fpr,"   * - TARGET  Host %s \n",inet_ntoa(Dtg.DtgDst));
			fprintf(fpr,"   * - SPOOFED Host %s \n",inet_ntoa(Dtg.DtgSrc));
			fprintf(fpr,"   * AND Spoofed Host Doesn't Exist\n");
            fprintf(fpr,"================< END >================ \n");
            fprintf(fpr,"\n");
		}
	
			
};

/*
void ShowList(FILE * dbgfpr)
{
	struct SrcNode *ShowSN;
	struct DstNode *ShowDN;
	struct in_addr ip_addr;
		
	ShowSN = CurList;
	
		
	fprintf(dbgfpr,"-------- H LISTA ARXH --------\n");
	while (ShowSN != NULL)
	{
		ShowDN = ShowSN->FirstDN;
		ip_addr = ShowSN->Src; 
		fprintf(dbgfpr,"SRC : %s \n",inet_ntoa(ip_addr));
		while (ShowDN != NULL)
		{
			ip_addr = ShowDN->Dst;
			fprintf(dbgfpr," ---------> DST %s , TIME %ld \n",inet_ntoa(ip_addr),ShowDN->T);
			ShowDN=ShowDN->NextDN;
		}
		ShowSN=ShowSN->NextSN;
	}
	fprintf(dbgfpr,"--------------- END ------------ \n");
	
			
};*/

/*void GlobalDelete()
{
    if (CurList !=NULL)
    {
        DelSN = CurList;
        
        while(DelSN->NextSN !=NULL)
        {
            DelDN = DelSN->FirstDN;       
            while (DelDN->NextDN !=NULL)
            {
                DelDN=DelDN->NextDN;
                free(DelDN->PrevDN);
            }
            free(DelDN);
        
            DelSN=DelSN->NextSN;
            free(DelSN->PrevSN);
        }
    
        DelDN=DelSN->FirstDN;
    
        while (DelDN->NextDN != NULL)
        {
            DelDN=DelDN->NextDN;
            free(DelDN->PrevDN);
        }
        free(DelDN);
        free(DelSN);
    }
    
    if (TrBin.TrSN!=NULL)
    {
        InsSN = TrBin.TrSN;
        while (InsSN->NextSN != NULL)
        {
            InsSN=InsSN->NextSN;
            free(InsSN->PrevSN);
        }
        free(InsSN);
    }
    
    
    if (TrBin.TrDN!=NULL)
    {
        InsDN = TrBin.TrDN;
        while (InsDN->NextDN != NULL)
        {
            InsDN = InsDN->NextDN;
            free(InsDN->PrevDN);
        }
        free(InsDN);
    }
};*/

/*void PreprocCleanExitFunction(int signal)
{
        clean exit code goes here
}

void PreprocRestartFunction(int signal)
{
        restart code goes here
}
                               */
